import nunjucks from "nunjucks"
import toml from "toml";
import fs from "fs";
import path from "path";
import { program } from "commander";
import express from "express";
import livereload from "livereload"

import { hide, unhide } from "./hide.js";

const url_path      = "/";
const template_dir  = "template/";
const template_path = "index.html.njk";
const output_dir   = "dist";
const output_path   = "dist/index.html";
var config_path   = "config.toml";

function main() {
  program
    .option("--serve", "serve the gerneated files")
    .option("--generate", "generate prod ready files")
    .option("--config <config>", "config file you use, config-example.toml as example");
  program.parse();
  const options = program.opts();

  if (options.serve && options.generate
      || ! (options.serve || options.generate)) {
    program.help();
  }

  if (options.config) {
    config_path = options.config;
  }

  let config = fs.readFileSync(config_path, {encoding: "utf-8"});
  config = toml.parse(config)

  for (let i = 0; i < config["section"].length; i++) {
    for (let j = 0; j < config["section"][i]["items"].length; j++) {
      let item = config["section"][i]["items"][j];
      if (item.hide_permalink) {
        let seed = Math.floor(Math.random() * 1000000007)
        item.permalink = hide(seed, item.permalink);
        item.seed = seed;
      }
    }
  }

  let nunjucks_env = nunjucks.configure(template_dir, {
    autoescape: true,
    throwOnUndefined: true,
    trimBlocks: true,
    lstripBlocks: true,
  });

  if (options.serve) {
    serve(config, nunjucks_env);
  }

  if (options.generate) {
    generate(config, nunjucks_env);
  }
};

function serve(config, nunjucks_env) {
  config.dev = true;
  generate(config, nunjucks_env);

  const regen = function(event, filename) {
    console.log(`got ${event} event from ${filename}, regenerating...`);
    generate(config, nunjucks_env);
  };
  fs.watch(template_dir, {}, regen);
  fs.watch(config_path, {}, regen);

  const app = express();
  const port = 3000;
  app.use(express.static(output_dir));
  app.listen(port, () => {
    console.log(`serveing webpage at ${port}`);
  });

  const lrserver = livereload.createServer();
  lrserver.watch(output_dir);
}

function generate(config, nunjucks_env) {
  const rendered = nunjucks_env.render(template_path, config);
  if (! fs.existsSync(path.dirname(output_path))) {
    fs.mkdirSync(path.dirname(output_path), { recursive: true });
  }
  fs.writeFileSync(output_path, rendered);
}


main();
